@@ -87,6 +87,9 @@ struct _GstVaapiDecoderVp9Class
   GstVaapiDecoderClass parent_class;
 };
 
+G_DEFINE_TYPE (GstVaapiDecoderVp9, gst_vaapi_decoder_vp9,
+    GST_TYPE_VAAPI_DECODER);
+
 static GstVaapiDecoderStatus
 get_status (GstVp9ParserResult result)
 {
@@ -150,6 +153,15 @@ gst_vaapi_decoder_vp9_create (GstVaapiDecoder * base_decoder)
   return TRUE;
 }
 
+static GstVaapiDecoderStatus
+gst_vaapi_decoder_vp9_reset (GstVaapiDecoder * base_decoder)
+{
+  gst_vaapi_decoder_vp9_destroy (base_decoder);
+  if (gst_vaapi_decoder_vp9_create (base_decoder))
+    return GST_VAAPI_DECODER_STATUS_SUCCESS;
+  return GST_VAAPI_DECODER_STATUS_ERROR_UNKNOWN;
+}
+
 /* Returns GstVaapiProfile from VP9 frame_hdr profile value */
 static GstVaapiProfile
 get_profile (guint profile_idc)
@@ -170,7 +182,7 @@ get_profile (guint profile_idc)
       profile = GST_VAAPI_PROFILE_VP9_3;
       break;
     default:
-      g_debug ("unsupported profile_idc value");
+      GST_DEBUG ("unsupported profile_idc value");
       profile = GST_VAAPI_PROFILE_UNKNOWN;
       break;
   }
@@ -729,18 +741,24 @@ gst_vaapi_decoder_vp9_flush (GstVaapiDecoder * base_decoder)
   return GST_VAAPI_DECODER_STATUS_SUCCESS;
 }
 
+static void
+gst_vaapi_decoder_vp9_finalize (GObject * object)
+{
+  GstVaapiDecoder *const base_decoder = GST_VAAPI_DECODER (object);
+
+  gst_vaapi_decoder_vp9_destroy (base_decoder);
+  G_OBJECT_CLASS (gst_vaapi_decoder_vp9_parent_class)->finalize (object);
+}
+
 static void
 gst_vaapi_decoder_vp9_class_init (GstVaapiDecoderVp9Class * klass)
 {
-  GstVaapiMiniObjectClass *const object_class =
-      GST_VAAPI_MINI_OBJECT_CLASS (klass);
+  GObjectClass *const object_class = G_OBJECT_CLASS (klass);
   GstVaapiDecoderClass *const decoder_class = GST_VAAPI_DECODER_CLASS (klass);
 
-  object_class->size = sizeof (GstVaapiDecoderVp9);
-  object_class->finalize = (GDestroyNotify) gst_vaapi_decoder_finalize;
+  object_class->finalize = gst_vaapi_decoder_vp9_finalize;
 
-  decoder_class->create = gst_vaapi_decoder_vp9_create;
-  decoder_class->destroy = gst_vaapi_decoder_vp9_destroy;
+  decoder_class->reset = gst_vaapi_decoder_vp9_reset;
   decoder_class->parse = gst_vaapi_decoder_vp9_parse;
   decoder_class->decode = gst_vaapi_decoder_vp9_decode;
   decoder_class->start_frame = gst_vaapi_decoder_vp9_start_frame;
@@ -748,17 +766,12 @@ gst_vaapi_decoder_vp9_class_init (GstVaapiDecoderVp9Class * klass)
   decoder_class->flush = gst_vaapi_decoder_vp9_flush;
 }
 
-static inline const GstVaapiDecoderClass *
-gst_vaapi_decoder_vp9_class (void)
+static void
+gst_vaapi_decoder_vp9_init (GstVaapiDecoderVp9 * decoder)
 {
-  static GstVaapiDecoderVp9Class g_class;
-  static gsize g_class_init = FALSE;
+  GstVaapiDecoder *const base_decoder = GST_VAAPI_DECODER (decoder);
 
-  if (g_once_init_enter (&g_class_init)) {
-    gst_vaapi_decoder_vp9_class_init (&g_class);
-    g_once_init_leave (&g_class_init, TRUE);
-  }
-  return GST_VAAPI_DECODER_CLASS (&g_class);
+  gst_vaapi_decoder_vp9_create (base_decoder);
 }
 
 /**
@@ -774,5 +787,6 @@ gst_vaapi_decoder_vp9_class (void)
 GstVaapiDecoder *
 gst_vaapi_decoder_vp9_new (GstVaapiDisplay * display, GstCaps * caps)
 {
-  return gst_vaapi_decoder_new (gst_vaapi_decoder_vp9_class (), display, caps);
+  return g_object_new (GST_TYPE_VAAPI_DECODER_VP9, "display", display,
+      "caps", caps, NULL);
 }
