@@ -1511,9 +1511,7 @@ fill_sequence (GstVaapiEncoderH265 * encoder, GstVaapiEncSequence * sequence)
 
   seq_param->general_profile_idc = encoder->profile_idc;
   seq_param->general_level_idc = encoder->level_idc;
-  seq_param->general_tier_flag = 0;     /* FIXME: use the tier flag
-                                         * extracted from upstream
-                                         * caps or calcuted one */
+  seq_param->general_tier_flag = encoder->tier;
 
   seq_param->intra_period = GST_VAAPI_ENCODER_KEYFRAME_PERIOD (encoder);
   seq_param->intra_idr_period = encoder->idr_period;
@@ -1867,10 +1865,13 @@ ensure_control_rate_params (GstVaapiEncoderH265 * encoder)
   GST_VAAPI_ENCODER_VA_RATE_CONTROL (encoder).window_size = encoder->cpb_length;
   GST_VAAPI_ENCODER_VA_RATE_CONTROL (encoder).initial_qp = encoder->init_qp;
   GST_VAAPI_ENCODER_VA_RATE_CONTROL (encoder).min_qp = encoder->min_qp;
+
+#if VA_CHECK_VERSION(1,1,0)
   /* @FIXME: should not set this value, should be ignored if set to zero *
    * https://github.com/intel/media-driver/issues/587 */
   if (encoder->min_qp > 0)
     GST_VAAPI_ENCODER_VA_RATE_CONTROL (encoder).max_qp = 51;
+#endif
 
 #if VA_CHECK_VERSION(1,0,0)
   GST_VAAPI_ENCODER_VA_RATE_CONTROL (encoder).rc_flags.bits.mb_rate_control =
