@@ -181,7 +181,7 @@ gst_vaapiencode_default_set_property (GstVaapiEncode * encode, guint prop_id,
     g_value_copy (value, &prop_value->value);
 
     if (encoder)
-      return (gst_vaapi_encoder_set_property (encoder, prop_id,
+      return (gst_vaapi_encoder_set_property (encoder, prop_value->id,
               value) == GST_VAAPI_ENCODER_STATUS_SUCCESS);
 
     return TRUE;
