@@ -1284,6 +1284,11 @@ gst_vaapipostproc_transform_meta (GstBaseTransform * trans, GstBuffer * outbuf,
   /* dont' GstVideoCropMeta if use_vpp */
   if (meta->info->api == GST_VIDEO_CROP_META_API_TYPE && postproc->use_vpp)
     return FALSE;
+
+  /* don't copy GstParentBufferMeta if use_vpp */
+  if (meta->info->api == GST_PARENT_BUFFER_META_API_TYPE && postproc->use_vpp)
+    return FALSE;
+
   return TRUE;
 }
 
@@ -1313,9 +1318,7 @@ gst_vaapipostproc_transform (GstBaseTransform * trans, GstBuffer * inbuf,
   ret = GST_FLOW_NOT_SUPPORTED;
   if (postproc->flags) {
     /* Use VA/VPP extensions to process this frame */
-    if (postproc->has_vpp &&
-        (postproc->flags != GST_VAAPI_POSTPROC_FLAG_DEINTERLACE ||
-            deint_method_is_advanced (postproc->deinterlace_method))) {
+    if (postproc->has_vpp) {
       ret = gst_vaapipostproc_process_vpp (trans, buf, outbuf);
       if (ret != GST_FLOW_NOT_SUPPORTED)
         goto done;
@@ -1715,8 +1718,7 @@ gst_vaapipostproc_class_init (GstVaapiPostprocClass * klass)
   element_class->set_context = gst_vaapi_base_set_context;
   gst_element_class_set_static_metadata (element_class,
       "VA-API video postprocessing",
-      "Filter/Converter/Video;Filter/Converter/Video/Scaler;"
-      "Filter/Effect/Video;Filter/Effect/Video/Deinterlace",
+      "Filter/Converter/Effect/Video/Scaler/Deinterlace/Hardware",
       GST_PLUGIN_DESC, "Gwenole Beauchesne <gwenole.beauchesne@intel.com>");
 
   /* sink pad */
